@@ -94,7 +94,26 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
 
             this->dmtx_thread.hasFoundBarcode = false; // reset this so we don't look at this data again
 
-            emit imageAnalysed(TYPE_DMTX, QString::fromUtf8(this->dmtx_thread.thread_output)); // *output
+            const uchar *buffer = (uchar *)this->dmtx_thread.thread_buffer;
+            unsigned int height = this->dmtx_thread.frame_height;
+            unsigned int width = this->dmtx_thread.frame_width;
+            QImage::Format format = QImage::Format_Indexed8; // we can't always assume this.....
+            QImage *qi = new QImage(width, height, format); //(buffer, width, height, width, format);
+            // generate a colourmap
+            qi->setNumColors(256);
+            for(int i=0;i<256;i++){
+                qi->setColor(i,QColor(i,i,i,255).rgba());
+            }
+            qi->fill(0);
+            for(unsigned int i=0;i<width;i++){
+                for(unsigned int j=0;j<height;j++){
+                    qi->setPixel(i,j,buffer[i+j*width]);
+                }
+            }
+
+            emit imageAnalysed(TYPE_DMTX,
+                               QString::fromUtf8(this->dmtx_thread.thread_output),
+                               qi); // *output
 
             return ZBAR_QRCODE; // QR is also 2D so achieves the same effect //BARCODE_TYPE_DMTX;
 
@@ -146,7 +165,27 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
             qDebug() << "analyse_image(): found ZBar barcode, zbar_thread.thread_output=" << QString(this->zbar_thread.thread_output);
             qDebug() << "analyse_image(): found ZBar barcode, zbar_thread.barcode_type=" << QString(this->zbar_thread.barcode_type);
 
-            emit imageAnalysed(QString::fromUtf8(this->zbar_thread.barcode_type),QString::fromUtf8(this->zbar_thread.thread_output)); // *output
+            const uchar *buffer = (uchar *)this->zbar_thread.thread_buffer;
+            unsigned int height = this->zbar_thread.frame_height;
+            unsigned int width = this->zbar_thread.frame_width;
+            QImage::Format format = QImage::Format_Indexed8; // we can't always assume this.....
+            QImage *qi = new QImage(width, height, format); //(buffer, width, height, width, format);
+            // generate a colourmap
+            qi->setNumColors(256);
+            for(int i=0;i<256;i++){
+                qi->setColor(i,QColor(i,i,i,255).rgba());
+            }
+            qi->fill(0);
+            for(unsigned int i=0;i<width;i++){
+                for(unsigned int j=0;j<height;j++){
+                    qi->setPixel(i,j,buffer[i+j*width]);
+                }
+            }
+
+
+            emit imageAnalysed(QString::fromUtf8(this->zbar_thread.barcode_type),
+                               QString::fromUtf8(this->zbar_thread.thread_output),
+                               qi); // *output
 
             return 1; // not sure this makes any odds anymore anyway.
 
