@@ -62,25 +62,37 @@ MainWindow::MainWindow(QWidget *parent)
 {
     qDebug() << "MainWindow::MainWindow(): Setting up UI";
     ui->setupUi(this);
+
     qDebug() << "In MainWindow::MainWindow";
     settings = new QSettings("mbarcode", "mbarcode"); // load settings
+
     qDebug() << "MainWindow::MainWindow(): loading tillRollModel";
     tillRollModel = new QStandardItemModel(0,1);
     ui->tblTillRoll->setModel(tillRollModel);
+
     qDebug() << "MainWindow::MainWindow(): Initializing decoder";
     decoder = new BarcodeDetector(); // create a decoder
+
     qDebug() << "MainWindow::MainWindow(): Initializing results window";
     resultsWindow = new ResultsWindow(this); // create a resultswindow
+
     // set up the videoWidget
     qDebug() << "MainWindow::MainWindow(): Initializing video widget";
     videoWidget = new VideoWidget(this, decoder);
     videoWidget->setAttribute(Qt::WA_NativeWindow, true);
+    videoWidget->setContinuousFocusTime((int)(settings->value("autofocus_delay",1.5).toFloat()*1000)); // pull from the settings
     ui->videoContainer->addWidget(videoWidget); // add the widget to the container
+
     // connect the signal from the barcode decoder
-    connect(decoder, SIGNAL(imageAnalysed(QString, QString)),
-            this, SLOT(imageAnalysed(QString, QString)));
+    connect(decoder, SIGNAL(imageAnalysed(QString, QString, QImage*)),
+            this, SLOT(imageAnalysed(QString, QString, QImage*)));
     // connect signals to disable/enable the refocus button
     connect(videoWidget, SIGNAL(focusingValueChanged(bool)), this, SLOT(setFocusing(bool)));
+
+
+    connect(videoWidget, SIGNAL(doSaveBuffer()), this, SLOT(saveBufferImage()));
+
+
     qDebug() << "MainWindow::MainWindow(): Setting tblTillRoll properties";
     ui->tblTillRoll->setProperty("FingerScrollable", true); // apparently this will change for 4.6
     //ui->tillRoll->setItemDelegate(new BarcodeTableItemDelegate);
@@ -88,15 +100,21 @@ MainWindow::MainWindow(QWidget *parent)
 
     ui->tblTillRoll->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
     ui->tblTillRoll->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
+
+    qDebug() << "MainWindow::MainWindow(): Loading tillroll history";
+    loadHistory();
+
     qDebug() << "MainWindow::MainWindow(): Loading plugins";
     // load all plugins
-    loadPlugins();
+    loadPlugins(); // !!!change this to only load those specified in the settings class!!!
+
     qDebug() << "MainWindow::MainWindow(): Initializing buttons";
     // init our buttons
     ui->btnScan->setVisible(true);
     ui->btnCancel->setVisible(false);
     ui->btnRefocus->setVisible(false);
     ui->btnOpenImage->setVisible(true);
+
     bool fullscreenScan = settings->value("fullscreen_scan", false).toBool();
     if(fullscreenScan) {
         ui->tblTillRoll->setVisible(true);
@@ -109,11 +127,25 @@ MainWindow::MainWindow(QWidget *parent)
     qDebug() << "MainWindow::MainWindow(): Finished";
     setAttribute(Qt::WA_Maemo5StackedWindow);
     setAttribute(Qt::WA_Maemo5AutoOrientation, true); // currently not working since libqt4-maemo5 is broken. Will work in PR1.2
+
+    conn = dbus_bus_get(DBUS_BUS_SESSION, 0);
+
 }
 
 MainWindow::~MainWindow()
 {
+    saveHistory();
     delete ui;
+
+    // clean up any temp files
+    QDir d("/home/user/.mbarcode/temp/");
+    if(!d.exists("/home/user/.mbarcode/temp/"))
+        return;
+
+    foreach ( QString file, d.entryList( QDir::Files))
+        QFile::remove(QFileInfo( d, file).absoluteFilePath());
+
+
 }
 
 void MainWindow::loadPlugins() {
@@ -161,6 +193,79 @@ void MainWindow::loadPlugins() {
      resultsWindow->setSinkPlugins(sinkPlugins); // let the resultwindow have all our plugins
 }
 
+void MainWindow::loadHistory() {
+    // load the history list from the settings class data
+
+    if(!settings->value("save_history",true).toBool())
+        return;
+
+    int size = settings->beginReadArray("scan_history");
+
+    qDebug() << "MainWindow::loadHistory(): " << size << " items.";
+
+    for (int i = 0; i < size; ++i) {
+         settings->setArrayIndex(i);
+         QVariantHash itemData = settings->value("barcodeDataHash").toHash();
+
+         QStandardItem *barcodeItem = new QStandardItem(itemData.value("barcodeData").toString());
+
+         barcodeItem->setSizeHint(QSize(300,200)); // make the image 300x200
+
+         tillRollModel->insertRow(0); // insert at top
+         tillRollModel->setItem(0, 0, barcodeItem);
+
+
+         barcodeItem->setData(itemData);
+
+         // create a widget to hold an image and text
+         QWidget *w = new QWidget;
+         w->setAutoFillBackground(true); // otherwise we'll see bits of the text contents
+         QVBoxLayout *lay = new QVBoxLayout();
+         QLabel *imlabel = new QLabel;
+         imlabel->setPixmap(itemData.value("barcodeImage").value<QPixmap>());
+         lay->addWidget(imlabel);
+         QLabel *txtlabel = new QLabel(itemData.value("barcodeData").toString());
+         lay->addWidget(txtlabel);
+         w->setLayout(lay);
+
+         ui->tblTillRoll->setIndexWidget(barcodeItem->index(),w);
+
+
+     }
+     settings->endArray();
+
+}
+
+void MainWindow::saveHistory() {
+    // save the history list to the settings class data
+
+    if(!settings->value("save_history",true).toBool())
+        return;
+
+    qDebug() << "MainWindow::saveHistory(): " << tillRollModel->rowCount() << " items.";
+
+    settings->beginWriteArray("scan_history");
+    for (int i = 0; i < tillRollModel->rowCount() ; i++) {
+        QStandardItem *item = tillRollModel->item(i,0); // assumes we always use rows!
+        QVariantHash itemData = item->data().toHash();
+        //QString barcodeType = itemData.value("barcodeType").toString();
+        //QString barcodeData = itemData.value("barcodeData").toString();
+        //QIcon barcodeImage = tillRollModel->item(i)->icon();
+
+        settings->setArrayIndex(i);
+
+        settings->setValue("barcodeDataHash", itemData);
+        //settings->setValue("barcodeData", barcodeData);
+        //settings->setValue("barcodeImage", barcodeImage);
+
+
+     }
+     settings->endArray();
+
+     settings->sync();
+     qDebug() << "Settings error code: " << settings->status();
+}
+
 void MainWindow::on_btnScan_clicked()
 {
     startScan();
@@ -218,7 +323,7 @@ void MainWindow::dataAnalysed(QString datatype, QString barcodeData, QHash<QStri
     emit dataAnalysedSignal(datatype, barcodeData, data);
 }
 
-void MainWindow::imageAnalysed(QString barcodeType, QString barcodeData) {
+void MainWindow::imageAnalysed(QString barcodeType, QString barcodeData, QImage *im) {
     // this function is supposed to remove all previous results from the list (or at least all but the last three or something)
     // TODO: Fix the function so that is works...
 
@@ -229,45 +334,32 @@ void MainWindow::imageAnalysed(QString barcodeType, QString barcodeData) {
     // display the output (this is done before anything is added to the tblTillRoll - avoids flickering)
     showResultsWindow(barcodeData);
 
-    qDebug() << "MainWindow::imageAnalysed(): tillRollModel->rowCount()=" << tillRollModel->columnCount();
+    qDebug() << "MainWindow::imageAnalysed(): tillRollModel->rowCount()=" << tillRollModel->rowCount();
+
 
     // see how many rows are in our table first
     if (tillRollModel->rowCount()>SCAN_HISTORY_LENGTH){
         // too many, so remove one
         qDebug("MainWindow::imageAnalysed(): Table too long, removing one item");
-        tillRollModel->removeRow ( 0 );
+        tillRollModel->removeRow ( tillRollModel->rowCount()-1 );
     }
 
 
-    // then we generate a TableItem and a TableItemWidget to render the data
-    QStandardItem *barcodeItem = new QStandardItem(barcodeData);
-
-    qDebug() << "MainWindow::imageAnalysed(): Created new item " << barcodeData;
-
-
-    // now add our new data to the table
-    //ui->tillRoll->insertColumn(ui->tillRoll->columnCount());
-    //ui->tillRoll->setItem(0, ui->tillRoll->columnCount()-1, i);
-    tillRollModel->insertRow(0); // insert at top
-    tillRollModel->setItem(0, 0, barcodeItem);
-
-
-    // create a widget to hold an image and text
-    QWidget *w = new QWidget;
-    w->setAutoFillBackground(true); // otherwise we'll see bits of the text contents
-
-    QImage *im = decoder->getLastImage();
+    //QImage *im = decoder->getLastImage();
     QPixmap pm = QPixmap::fromImage(*im, Qt::AutoColor);
 
+    delete(im);
+
     pm = pm.scaledToWidth(300); // scale the image to 300 px wide
+
     // IMPORTANT: To change the size, you must also change the iconSize in mainwindow.ui for the table
-    QIcon icon = QIcon(pm);
+    //QIcon icon = QIcon(pm);
 
     tillRollModel->insertRow(0); // insert at top
 
-    QStandardItem *imageItem = new QStandardItem(icon, "");
-    imageItem->setSizeHint(QSize(300,200)); // make the image 300x200
-    tillRollModel->setItem(0, 0, imageItem);
+    QStandardItem *barcodeItem = new QStandardItem(barcodeData);
+    barcodeItem->setSizeHint(QSize(300,200)); // make the image 300x200
+    tillRollModel->setItem(0, 0, barcodeItem);
 
     qDebug() << "MainWindow::barcodeAnalysed(): Inserted new item. tillRollModel->rowCount()=" << tillRollModel->rowCount();
 
@@ -276,9 +368,24 @@ void MainWindow::imageAnalysed(QString barcodeType, QString barcodeData) {
     QVariantHash itemData;
     itemData.insert("barcodeType", barcodeType);
     itemData.insert("barcodeData", barcodeData);
+    itemData.insert("barcodeImage", pm);
 
-    imageItem->setData(itemData);
     barcodeItem->setData(itemData);
+
+
+    // create a widget to hold an image and text
+    QWidget *w = new QWidget;
+    w->setAutoFillBackground(true); // otherwise we'll see bits of the text contents
+    QVBoxLayout *lay = new QVBoxLayout();
+    QLabel *imlabel = new QLabel;
+    imlabel->setPixmap(pm);
+    lay->addWidget(imlabel);
+    QLabel *txtlabel = new QLabel(barcodeData);
+    lay->addWidget(txtlabel);
+    w->setLayout(lay);
+
+    ui->tblTillRoll->setIndexWidget(barcodeItem->index(),w);
+
 }
 
 void MainWindow::showResultsWindow(QString barcode) {
@@ -343,3 +450,85 @@ void MainWindow::on_actionTest_portrait_mode_triggered()
         settings->setValue("portrait_mode", true);
     }
 }
+
+// we should also allow single items to be deleted using e.g. a long press + context pop-up menu
+void MainWindow::on_actionClear_History_triggered()
+{
+    while (tillRollModel->rowCount()){
+        tillRollModel->removeRow ( tillRollModel->rowCount()-1 ); // remove last one
+    }
+}
+
+void MainWindow::on_tblTillRoll_customContextMenuRequested(QPoint pos)
+{
+    QAction *delAct = new QAction(tr("Delete item"), this);
+    connect(delAct, SIGNAL(triggered()), this, SLOT(delItem()));
+
+    QAction *saveAct = new QAction(tr("Save item"), this);
+    connect(saveAct, SIGNAL(triggered()), this, SLOT(saveItem()));
+
+
+    QMenu menu(this);
+    menu.addAction(delAct);
+    menu.addAction(saveAct);
+    menu.exec(QWidget::mapToGlobal(pos));
+
+}
+
+void MainWindow::delItem()
+{
+    // delete the current tillroll item
+    tillRollModel->removeRow ( ui->tblTillRoll->currentIndex().row() );
+}
+
+void MainWindow::saveItem()
+{
+    // save the current tillroll item to file
+    QStandardItem *item = tillRollModel->itemFromIndex(ui->tblTillRoll->currentIndex());
+
+    // save this file
+    // pop-up a save dialog
+    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"),
+                                "/home/user/MyDocs/.images/barcode.jpg",
+                                tr("JPEG Image (*.jpg)"));
+
+    QFileInfo finfo(fileName);
+    //if (finfo.exists())
+    //    return; // should at least offer a new filename I suppose...
+
+    // see if the extension survived....
+    if(finfo.completeSuffix().isEmpty())
+        fileName.append(".jpg");
+
+    QVariantHash itemData = item->data().toHash();
+    QPixmap pm = itemData.value("barcodeImage").value<QPixmap>();
+    QImage im = pm.toImage().convertToFormat(QImage::Format_RGB32);
+    im.save(fileName, 0, 95);
+
+
+}
+
+void MainWindow::saveBufferImage()
+{
+    this->cancelScan();
+
+    QImage *im = decoder->getLastImage();
+    im->convertToFormat(QImage::Format_RGB32);
+
+    // save this file
+    // pop-up a save dialog
+    QString fileName = QFileDialog::getSaveFileName(this, tr("Save current image buffer to file"),
+                                "/home/user/MyDocs/.images/barcode.jpg",
+                                tr("JPEG Image (*.jpg)"));
+
+    QFileInfo finfo(fileName);
+    //if (finfo.exists())
+    //    return; // should at least offer a new filename I suppose...
+
+    // see if the extension survived....
+    if(finfo.completeSuffix().isEmpty())
+        fileName.append(".jpg");
+
+    im->save(fileName, 0, 95);
+
+}
