@@ -22,6 +22,8 @@ private:
     Ui::SettingsWindow *ui;
 
 private slots:
+    void on_spnAutoFocusDelay_valueChanged(double );
+    void on_chkSaveHistory_clicked();
     void on_chkFullscreenScan_clicked();
 };
 
