@@ -43,9 +43,14 @@
 #include <QApplication>
 #include <QTime>
 #include <QTimer>
+
+#include <QFileDialog>
+#include <QFileInfo>
+
 // local includes
 #include "common.h"
 #include "barcodedetector.h"
+#include "mainwindow.h"
 
 VideoWidget *videoWidget_callback;
 VideoWidget::VideoWidget(QWidget *parent) : QWidget(parent) {
@@ -71,9 +76,12 @@ VideoWidget::VideoWidget(QWidget *parent, BarcodeDetector *decoder) : QWidget(pa
     getDeviceType(); // work out which device we are so we can specify what type of pipeline to setup
     initPipeline(); // create the GStreamer pipeline
     this->decoder = decoder; // create a new decoder object
-    QTimer *refocusTimer = new QTimer(this);
+
+    continuousFocusInterval = 1500;
+    refocusTimer = new QTimer(this);
     connect(refocusTimer, SIGNAL(timeout()), this, SLOT(refocus()));
-    refocusTimer->start(1500); // try to refocus every one-and-a-half seconds
+    refocusTimer->setInterval(continuousFocusInterval);
+
 }
 
 VideoWidget::~VideoWidget() {
@@ -113,6 +121,7 @@ void VideoWidget::start() {
 void VideoWidget::stop() {
     gst_element_set_state(pipeline, GST_STATE_NULL);
     pipelineReady = false;
+    refocusTimer->stop();
     focusing = false;
 }
 bool VideoWidget::initPipeline() {
@@ -513,12 +522,15 @@ void VideoWidget::refocus() {
         qDebug("VideoWidget::refocus(): Refocus called");
 //        gst_element_set_state (pipeline, GST_STATE_READY); // causes some problems with flickering, should probably be uncommented
         gst_element_set_state (pipeline, GST_STATE_PLAYING);
+        refocusTimer->start(); // try to refocus using the pre-set interval
     } else {
         qDebug("VideoWidget::refocus(): Already attempting autofocus or pipeline not ready... Please wait :)");
     }
 }
 void VideoWidget::setFocusing(bool enabled) {
     focusing = enabled;
+    if(!enabled)
+        refocusTimer->stop(); // stop the timer to save cycles
 }
 bool VideoWidget::isFocusing() {
     return focusing;
@@ -532,3 +544,19 @@ void VideoWidget::useFilesrc(QString filename) {
 //    g_object_set(G_OBJECT(input_selector), "active-pad", gst_element_get_pad(file_src, "src"), NULL); // make filesrc the source
 //    g_object_set(G_OBJECT(file_src), "location", filename.toAscii(), NULL); // set the location
 }
+
+void VideoWidget::setContinuousFocusTime(int milliseconds)
+{
+    if (refocusTimer->isActive()){
+        refocusTimer->stop();
+        refocusTimer->setInterval(milliseconds);
+        refocusTimer->start(); // try to refocus every one-and-a-half seconds
+    }else
+        refocusTimer->setInterval(milliseconds);
+
+}
+
+void VideoWidget::mousePressEvent(QMouseEvent *event){
+
+    emit doSaveBuffer();
+}
