@@ -62,6 +62,8 @@ public:
     bool isFocusing();
     void useVideosrc();
     void useFilesrc(QString filename);
+    void setContinuousFocusTime(int milliseconds);
+
 
 public slots:
     void refocus();
@@ -69,6 +71,8 @@ public slots:
 
 signals:
     void focusingValueChanged(bool enabled);
+    void doSaveBuffer();
+
 private:
     void setFocusing(bool enabled);
     GstElement *data, *pipeline;
@@ -105,6 +109,12 @@ private:
     zbar_image_scanner_t *scanner;
     zbar_image_t *image;
     bool focusing;
+
+    QTimer *refocusTimer;
+    int continuousFocusInterval;
+
+private slots:
+    void mousePressEvent(QMouseEvent *event);
 };
 
 #endif // VIDEOWIDGET_H
